@@ -63,11 +63,14 @@ impl<F: 'static + Float + std::ops::AddAssign> TransposedConvolutionLayer<F> {
 /// This implementation doesn't just assign the matrix patch-wise to a new
 /// matrix, but also sums the patches such that it unrolls the result
 /// of conv-transpose.
+///
 /// mat is a matrix of size (image_width * image_height, c_out * k_h * k_w),
 /// in which each entry is the value of a kernel pixel multiplied with an image pixels
 /// with the channels summed out
-/// The output naming is a bit confusing, as this function is originally used in the backward
-/// pass. What this function returns is simply (channels, height, width).
+///
+/// The naming is a bit confusing, as this function is originally used in the backward
+/// pass. What this function returns is simply an array of (channels, height, width).
+///
 fn col2im_pt<'a, F: 'a + Float + AddAssign>(
     data_col: &[F],
     channels: usize,
@@ -99,13 +102,14 @@ fn col2im_pt<'a, F: 'a + Float + AddAssign>(
                 let w_im = (w_col * stride) as i64 - pad_w as i64 + w_offset as i64;
 
                 if h_im < height as i64 && h_im >= 0 && w_im < width as i64 && w_im >= 0 {
-                    // TODO: Use unsafe get
-                    let el = res
-                        .get_mut((c_im * height + h_im as usize) * width + w_im as usize)
-                        .unwrap();
-
-                    // TODO: Use unsafe get
-                    el.add_assign(data_col[(c_col * height_col + h_col) * width_col + w_col]);
+                    unsafe {
+                        let el =
+                            res.uget_mut((c_im * height + h_im as usize) * width + w_im as usize);
+                        el.add_assign(
+                            *data_col
+                                .get_unchecked((c_col * height_col + h_col) * width_col + w_col),
+                        );
+                    }
                 }
             }
         }
@@ -113,13 +117,15 @@ fn col2im_pt<'a, F: 'a + Float + AddAssign>(
     Array::from_shape_vec((channels, height, width), res.into_iter().collect()).unwrap()
 }
 
-/// Performs a transposed convolution on the input image. This upsamples the image.
+/// Performs a transposed convolution on the input image. Might be used to upsample the image.
 /// This implementation is identical to the supplied by Pytorch and works by performing
 /// a "backwards pass" on the input.
 ///
 /// More explanation can be read here:
 /// - <https://datascience.stackexchange.com/questions/6107/what-are-deconvolutional-layers>
 ///
+/// The implementation is mostly taken from Pytorch.
+///
 /// NOTE: THERE IS A CHANGE IN KERNEL DIMENSIONS FOR CONV TRANSPOSED
 /// Input:
 /// -----------------------------------------------
@@ -156,7 +162,7 @@ where
     let im_h = im_mat.shape()[1];
     let im_w = im_mat.shape()[2];
     let im_reshape = im_mat.into_shape((im_c, im_h * im_w)).unwrap();
-    let data_matrix: Array2<F> = im_reshape.t().dot(&ker_reshape);
+    let data_matrix: Array2<F> = ker_reshape.t().dot(&im_reshape);
     // output padding might be necessary to achieve the desired shape
     let (pad_h, pad_w, output_pad_h, output_pad_w) = match padding {
         Padding::Valid => (0, 0, 0, 0),
@@ -164,11 +170,8 @@ where
     };
     let output_height = (im_h - 1) * stride + k_h + output_pad_h - 2 * pad_h;
     let output_width = (im_w - 1) * stride + k_w + output_pad_w - 2 * pad_w;
-    // let output_height = (im_h - 1) * stride + k_h;
-    // let output_width = (im_w - 1) * stride + k_w;
-    let data_matrix_contiguous = Array::from_iter(data_matrix.view().t().iter().map(|a| *a));
     let t_conv_output = col2im_pt(
-        data_matrix_contiguous.as_slice().unwrap(),
+        data_matrix.as_slice().unwrap(),
         k_f,
         output_height,
         output_width,
